﻿/* 
 * Capture Test NyARToolkitCSサンプルプログラム
 * --------------------------------------------------------------------------------
 * The MIT License
 * Copyright (c) 2008 nyatla
 * airmail(at)ebony.plala.or.jp
 * http://nyatla.jp/nyartoolkit/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * 
 */
using System;
using System.IO;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.Xml;
using System.Media;
using Microsoft.WindowsMobile.DirectX;
using Microsoft.WindowsMobile.DirectX.Direct3D;
using NyARToolkitCSUtils.NyAR;
using NyARToolkitCSUtils.Direct3d;
using jp.nyatla.nyartoolkit.cs;
using jp.nyatla.nyartoolkit.cs.core;
using jp.nyatla.nyartoolkit.cs.detector;
using jp.nyatla.nyartoolkit.cs.sandbox.quadx2;
using jp.nyatla.nyartoolkit.cs.sandbox.x2;
using jp.nyatla.cs.NyWMCapture;
using Microsoft.Samples.MD3DM;


namespace SimpleLiteDirect3d.WindowsMobile5
{
    public interface IMeshData
    {
        void Draw();
    }
    //テクスチャを共有するためのメッシュセット
    public class MeshData : IMeshData
    {
        public Texture[] m_textures = null;
        private Material[] m_meshMaterials = null;
        private Mesh m_mesh;
        private Device m_ref_dev;

        public MeshData(Device i_dev, String i_dir, String i_mesh_filename)
        {
            this.m_ref_dev = i_dev;
            string[] textureFilenames = null;
            // Ｘファイルを読み込んでメッシュを作成する
            this.m_mesh = MeshLoader.LoadMesh(i_dev,
                new FileStream(i_dir + i_mesh_filename, FileMode.Open, FileAccess.Read), MeshFlags.SystemMemory,
                out this.m_meshMaterials, out textureFilenames);
            this.m_textures = new Texture[this.m_meshMaterials.Length];
            for (int i = 0; i < this.m_meshMaterials.Length; i++)
            {
                // Set the ambient color for the material
                this.m_meshMaterials[i].Ambient = this.m_meshMaterials[i].Diffuse;
            }
            for (int i = 0; i < textureFilenames.Length; i++)
            {
                // Create the texture
                if (textureFilenames[i] != null)
                {
                    this.m_textures[i] = TextureLoader.FromStream(i_dev, new FileStream(i_dir + textureFilenames[i], FileMode.Open, FileAccess.Read));
                }
            }
            return;
        }
        public void Draw()
        {
            for (int i = 0; i < this.m_meshMaterials.Length; i++)
            {
                // マテリアルをセット
                this.m_ref_dev.Material = this.m_meshMaterials[i];
                // テクスチャーのセット
                this.m_ref_dev.SetTexture(0, this.m_textures[i]);
                // 描画
                this.m_mesh.DrawSubset(i);
            }
            return;
        }
    }
    public class SubMeshData : IMeshData
    {
        public Texture[] m_textures = null;
        private Material[] m_meshMaterials = null;
        private Mesh m_mesh;
        private Device m_ref_dev;

        public SubMeshData(Device i_dev, String i_dir, String i_mesh_filename, MeshData i_keymesh)
        {
            this.m_ref_dev = i_dev;
            string[] textureFilenames = null;
            // Ｘファイルを読み込んでメッシュを作成する
            this.m_mesh = MeshLoader.LoadMesh(i_dev,
                new FileStream(i_dir + i_mesh_filename, FileMode.Open, FileAccess.Read), MeshFlags.SystemMemory,
                out this.m_meshMaterials, out textureFilenames);
            this.m_textures = new Texture[this.m_meshMaterials.Length];
            for (int i = 0; i < this.m_meshMaterials.Length; i++)
            {
                // Set the ambient color for the material
                this.m_meshMaterials[i].Ambient = this.m_meshMaterials[i].Diffuse;
                // Create the texture
                this.m_textures[i] = i_keymesh.m_textures[i];
            }
            return;
        }
        public void Draw()
        {
            for (int i = 0; i < this.m_meshMaterials.Length; i++)
            {
                // マテリアルをセット
                this.m_ref_dev.Material = this.m_meshMaterials[i];
                // テクスチャーのセット
                this.m_ref_dev.SetTexture(0, this.m_textures[i]);
                // 描画
                this.m_mesh.DrawSubset(i);
            }
            return;
        }
    }
    public class MeshSet:IDisposable
    {
        private uint _draw_counter = 0;
        private IMeshData[] _mesh_data;
        //public MeshSet(D3dManager i_d3dmgr,ResourceBuilder i_builder)
        //{
        //    Device dev = i_d3dmgr.d3d_device;
        //    String root_path = i_builder.root_path;
        //    MeshData zero=new MeshData(dev, root_path + "\\miku\\", "01.md3dm");
        //    this._mesh_data[0] = zero;
        //    this._mesh_data[1] = new SubMeshData(dev, root_path + "\\miku\\", "02.md3dm", zero);
        //    this._mesh_data[2] = new SubMeshData(dev, root_path + "\\miku\\", "03.md3dm", zero);
        //    this._mesh_data[3] = new SubMeshData(dev, root_path + "\\miku\\", "04.md3dm", zero);
        //    this._mesh_data[4] = new SubMeshData(dev, root_path + "\\miku\\", "05.md3dm", zero);
        //    this._mesh_data[5] = new SubMeshData(dev, root_path + "\\miku\\", "06.md3dm", zero);
        //    this._mesh_data[6] = new SubMeshData(dev, root_path + "\\miku\\", "07.md3dm", zero);
        //    this._mesh_data[7] = new SubMeshData(dev, root_path + "\\miku\\", "08.md3dm", zero);
        //    this._mesh_data[8] = new SubMeshData(dev, root_path + "\\miku\\", "09.md3dm", zero);
        //    this._mesh_data[9] = new SubMeshData(dev, root_path + "\\miku\\", "10.md3dm", zero);
        //    this._mesh_data[10] = new SubMeshData(dev, root_path + "\\miku\\", "11.md3dm", zero);
        //    this._mesh_data[11] = new SubMeshData(dev, root_path + "\\miku\\", "12.md3dm", zero);
        //    return;
        //}
        public MeshSet(D3dManager i_d3dmgr, ResourceBuilder i_builder)
        {
            Device dev = i_d3dmgr.d3d_device;
            String root_path = i_builder.root_path;

                         
            //ファイル名の確定
            String xml_path;
            XmlDocument dom = new XmlDocument();
            if (i_builder.content_xml[0] != '\\')
            {
                xml_path = i_builder.root_path + "\\" + i_builder.content_xml;
            }else
            {
                xml_path = i_builder.content_xml;
            }
            dom.Load(xml_path);
            
            if (dom.SelectSingleNode("/root/version").InnerText != "Uma2MobileObject/0.1")
            {
                throw new Exception("object::設定ファイルのバージョンが違います？");
            }
            //バカだからタイプを確認しない

            //スケール取得
            float sc = float.Parse(dom.SelectSingleNode("/root/md3dmset/@scale").Value);
            this._scale = Matrix.Scaling(sc, sc, sc);

            //md3dmsetからメッシュリストを作る。
            XmlNodeList md3dm_nodes = dom.SelectNodes("/root/md3dmset/md3dm/@file");
            if (md3dm_nodes.Count < 1)
            {
                throw new Exception("MeshSet:has not md3dm node.");
            }
            this._mesh_data= new IMeshData[md3dm_nodes.Count];
            //xmlの有るパスを取得
            String md3dm_root_path = System.IO.Path.GetDirectoryName(xml_path);
            //
            MeshData zero = new MeshData(dev, md3dm_root_path + "\\",md3dm_nodes[0].Value);
            this._mesh_data[0] = zero;
            for (int i = 1; i < md3dm_nodes.Count; i++)
            {
                this._mesh_data[i] = new SubMeshData(dev, md3dm_root_path + "\\", md3dm_nodes[i].Value, zero);
            }
            return;
        }
        private Matrix _scale;
        public Matrix scale { get { return this._scale; } }

        public void Draw()
        {
            this._mesh_data[this._draw_counter % this._mesh_data.Length].Draw();
            this._draw_counter++;
            return;
        }
        public void Dispose()
        {
            for (int i = 0; i < this._mesh_data.Length - 1; i++)
            {
                if (this._mesh_data[i] != null)
                {
                    this._mesh_data[i] = null;
                }
            }
            return;
        }
    }

    public partial class SimpleLiteD3d : IDisposable,INySampleCB
    {
        private int _threshold = 110;
        private D3dManager _d3dmgr;
        private MeshSet _meshset;
        private ID3dBackground _back_ground;
        private WmCapture _capture;
        //NyAR
        private NyARSingleDetectMarker_Quad m_ar;
        private DsRGB565Raster m_raster;
        private NyARD3dUtil _utils = new NyARD3dUtil();
        private String _capture_image_dir;
        private int _capture_image_format;

        public SimpleLiteD3d(NyARToolkitCS topLevelForm, ResourceBuilder i_resource)
        {
            NyMath.initialize();
            this._capture = i_resource.createWmCapture();
            this._capture.setOnSample(this);

            this._d3dmgr = i_resource.createD3dManager(topLevelForm);
            this._back_ground = i_resource.createBackGround(this._d3dmgr);

            //AR用のパターンコードを読み出
            NyARCode code = i_resource.createNyARCode();

            //１パターンのみを追跡するクラスを作成
            this.m_ar = new NyARSingleDetectMarker_Quad(i_resource.ar_param, code, 80.0);
            //計算モードの設定
            this.m_ar.setContinueMode(false);

            this._utils = new NyARD3dUtil();

            //ARラスタを作る(DirectShowキャプチャ仕様)。
            this._meshset = new MeshSet(this._d3dmgr, i_resource);
            this.m_raster = i_resource.createARRaster();
            this._d3dmgr.d3d_device.DeviceLost += new EventHandler(OnDeviceLost);
            this._d3dmgr.d3d_device.DeviceReset += new EventHandler(OnDeviceReset);
            //キャプチャ情報の保管
            this._capture_image_dir = i_resource.capture_dir;
            this._capture_image_format = i_resource.capture_format;
            return;
        }
        private void OnDeviceLost(object sender, EventArgs e)
        {
            MessageBox.Show("OnDeviceLost");
            return;
        }
        private void OnDeviceReset(object sender, EventArgs e)
        {
            MessageBox.Show("OnDeviceReset");
            return;
        }
        private uint _capture_counter = 0;
        /* 非同期イベントハンドラ
         * CaptureDeviceからのイベントをハンドリングして、バッファとテクスチャを更新する。
         */
        public int OnSample(INySample i_sample)
        {
            //フレームスキップ
            if (this._capture_counter % 2 == 2)
            {
                return 0;
            }
            lock (this)
            {
                IntPtr data=i_sample.GetData();
                this.m_raster.setBuffer(data);
                //テクスチャ内容を更新
                this._back_ground.setSample(i_sample);
                //マーカーは見つかったかな？
                is_marker_enable = this.m_ar.detectMarkerLite(this.m_raster, this._threshold);
                if (is_marker_enable && this.m_ar.getConfidence() > 0.3)
                {
                    //あればMatrixを計算
                    this.m_ar.getTransmationMatrix(trans_result);
                    this._utils.toD3dMatrix(trans_result, ref trans_matrix);
                    this._lost_counter = 5;
                }
                else if (this._lost_counter > 0)
                {
                    is_marker_enable = true;
                    this._lost_counter--;
                }
                else
                {
                    is_marker_enable = false;
                }
            }
            Thread.Sleep(10);
            this._capture_counter++;
            return 0;
        }
        /* キャプチャを開始する関数
         */
        public void start()
        {
            this._capture.start();
        }
        /* キャプチャを停止する関数
         */
        public void stop()
        {
            this._capture.stop();
        }
        //ファイルを保存する。
        public void saveImage()
        {
            System.Drawing.Imaging.ImageFormat fmt;
            String file_name = this._capture_image_dir + "\\UM_" + Convert.ToString(DateTime.Now.Ticks, 16);
            lock (this)
            {
                switch (this._capture_image_format)
                {
                    case ResourceBuilder.IMGFMT_BMP:
                        file_name += ".bmp";
                        fmt = System.Drawing.Imaging.ImageFormat.Bmp;
                        break;
                    case ResourceBuilder.IMGFMT_GIF:
                        file_name += ".gif";
                        fmt = System.Drawing.Imaging.ImageFormat.Gif;
                        break;
                    case ResourceBuilder.IMGFMT_JPG:
                        file_name += ".jpeg";
                        fmt = System.Drawing.Imaging.ImageFormat.Jpeg;
                        break;
                    case ResourceBuilder.IMGFMT_PNG:
                        file_name += ".png";
                        fmt = System.Drawing.Imaging.ImageFormat.Png;
                        break;
                    default:
                        throw new Exception("未知のフォーマット");
                }
                try
                {
                    SystemSounds.Hand.Play();
                    this._d3dmgr.saveFrontSurfaceToFile(file_name, fmt);
                }
                catch (Exception e)
                {
                    MessageBox.Show("ERROR:saveImage", e.Message);
                }
            }
            return;
        }
        public void setThreshold(int i_threshold)
        {
            lock (this)
            {
                this._threshold = i_threshold;
            }
            return;
        }

        private bool is_marker_enable=false;
        private Matrix trans_matrix = new Matrix();
        private NyARTransMatResult trans_result = new NyARTransMatResult();
        private int _lost_counter=0;//後何回連続ロスト出来るかを返す
        //メインループ処理
        public void MainLoop()
        {
            // 3Dオブジェクトの描画はここから
            this._d3dmgr.d3d_device.BeginScene();
            //ARの計算
            lock (this)
            {
                this._back_ground.drawBackGround();
                this._d3dmgr.d3d_device.Clear(ClearFlags.ZBuffer, System.Drawing.Color.Blue, 1.0f, 0);

                //マーカーが見つかっていて、0.3より一致してたら描画する。
                if (is_marker_enable)
                {

                    Matrix transform_mat2 = this._meshset.scale;
                    transform_mat2 *= Matrix.RotationX((float)Math.PI * 1 / 2);
                    //変換行列を掛ける
                    transform_mat2 *= trans_matrix;
                    // 計算したマトリックスで座標変換
                    this._d3dmgr.d3d_device.SetTransform(TransformType.World, transform_mat2);
                    //描画
                    this._meshset.Draw();
                }
                
                // 描画はここまで
                this._d3dmgr.d3d_device.EndScene();

                // 実際のディスプレイに描画*/
                this._d3dmgr.d3d_device.Present();
            }
            //スレッドスイッチ
            Thread.Sleep(10);
        }


        // リソースの破棄をするために呼ばれる
        public void Dispose()
        {
            //キャプチャ解除
            if (this._capture != null)
            {
                this._capture.Dispose();
            }
            if(this._meshset!=null){
                this._meshset.Dispose();
            }
            if (this._back_ground != null)
            {
                this._back_ground.Dispose();
            }
            // Direct3D デバイスのリソース解放
            if (this._d3dmgr != null)
            {
                this._d3dmgr.Dispose();
            }
        }



    }
}
